package com.build.caching;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import net.sf.ehcache.Cache;
import net.sf.ehcache.CacheManager;
import net.sf.ehcache.Element;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class CacheFacade {
	
	private Cache cache;
	private Random random;
	private final int numElements;
	private final int elementSize;
	
	private static final Log log = LogFactory.getLog(CacheFacade.class);
	
	public CacheFacade(String cacheName, int numElements, int elementSize) {
		random = new Random();
		cache = CacheManager.getInstance().getCache(cacheName);
		this.numElements = numElements;
		this.elementSize = elementSize;
	}
	
	public void put() {
		int cacheSize = cache.getKeys().size();
		if (cacheSize == numElements) {
			log.info(cache.getName() + " already fully loaded with " + numElements + " elements.");
		} else {
			log.info(cache.getName() + " has " + cacheSize + " elements, putting " + numElements + " byte arrays of size " + elementSize + " in cache");
			List<Element> elements = new ArrayList<Element>(numElements);
			for (int i = 0; i < numElements; i++) {
				byte[] b = new byte[elementSize];
				random.nextBytes(b);
				elements.add(new Element(i, b));
			}
			long before = System.currentTimeMillis();
			cache.putAll(elements);
			long elapsed = System.currentTimeMillis() - before;
			log.info(cache.getName() + " putAll for " + numElements + " elements took " + elapsed + " ms.");			
		}
	}
	
	public boolean get(int key) {
		long before = System.currentTimeMillis();
		Element element = cache.get(key);
		long elapsed = System.currentTimeMillis() - before;
		log.info(cache.getName() + " get call took " + elapsed + " ms.");
		
		if (element == null) {
			log.info(cache.getName() + " miss");
			return false;
		} else {
			log.info(cache.getName() + " hit");
			return true;
		}
	}

}
